@@ -353,7 +353,11 @@ class InputParameter(ArgoVar):
             description = jsonpickle.dumps({"type": str(self.type)})
 
         if self.save_as_artifact:
-            if hasattr(self, "value"):
+            if isinstance(self.source, (InputParameter, OutputParameter,
+                                        InputArtifact, OutputArtifact)):
+                return V1alpha1Artifact(name="dflow_bigpar_" + self.name,
+                                        path=self.path, _from=str(self.source))
+            elif hasattr(self, "value"):
                 if isinstance(self.value, (InputParameter, OutputParameter,
                                            InputArtifact, OutputArtifact)):
                     return V1alpha1Artifact(name="dflow_bigpar_" + self.name,
@@ -381,10 +385,6 @@ class InputParameter(ArgoVar):
                         return V1alpha1Artifact(
                             name="dflow_bigpar_" + self.name, path=self.path,
                             oss=s3.oss())
-            elif isinstance(self.source, (InputParameter, OutputParameter,
-                                          InputArtifact, OutputArtifact)):
-                return V1alpha1Artifact(name="dflow_bigpar_" + self.name,
-                                        path=self.path, _from=str(self.source))
             else:
                 return V1alpha1Artifact(name="dflow_bigpar_" + self.name,
                                         path=self.path)
@@ -427,7 +427,7 @@ class InputArtifact(ArgoVar):
             type: Optional[Any] = None,
             source: Union[str, "InputArtifact",
                           "OutputArtifact", S3Artifact] = None,
-            mode: Optional[int] = 0o755,
+            mode: Optional[int] = None,
             sub_path: Optional[str] = None,
             archive: str = "default",
     ) -> None:
@@ -663,7 +663,8 @@ class OutputParameter(ArgoVar):
             kwargs = {
                 "name": "dflow_bigpar_" + self.name,
                 "archive": V1alpha1ArchiveStrategy(_none={}),
-                "global_name": self.global_name,
+                "global_name": "dflow_bigpar_" + self.global_name if
+                self.global_name is not None else None,
             }
             if self.value_from_path is not None:
                 return V1alpha1Artifact(path=self.value_from_path, **kwargs)
